@@ -8,7 +8,7 @@ use base qw/App::CLI::Command Class::Accessor::Fast Shipwright::Script/;
 __PACKAGE__->mk_accessors(qw/all follow builder utility version/);
 
 use Shipwright;
-use File::Spec::Functions qw/catdir/;
+use File::Spec;
 use Shipwright::Util;
 use File::Copy qw/copy move/;
 use File::Temp qw/tempdir/;
@@ -44,7 +44,7 @@ sub run {
     }
     else {
 
-        confess "need name arg\n" unless $name || $self->all;
+        die "need name arg\n" unless $name || $self->all;
 
         $map    = $shipwright->backend->map    || {};
         $source = $shipwright->backend->source || {};
@@ -80,12 +80,12 @@ sub run {
                 @dists = $name;
             }
 
-            for (@dists) {
+            for ( @dists ) {
                 if ( $_ eq $name ) {
                     $self->_update( $_, $self->version );
                 }
                 else {
-                    $self->_update($_);
+                    $self->_update( $_ );
                 }
             }
         }
@@ -121,7 +121,7 @@ sub _update {
             $name = $map->{$name};
         }
         else {
-            confess 'invalid name ' . $name . "\n";
+            die 'invalid name ' . $name . "\n";
         }
 
         $shipwright->source(
@@ -138,8 +138,8 @@ sub _update {
     $version = Shipwright::Util::LoadFile( $shipwright->source->version_path );
 
     $shipwright->backend->import(
-        source    => catdir( $shipwright->source->directory, $name ),
-        comment   => "update $name",
+        source  => File::Spec->catfile( $shipwright->source->directory, $name ),
+        comment => "update $name",
         overwrite => 1,
         version   => $version->{$name},
     );
