@@ -3,17 +3,19 @@ package Shipwright::Source::CPAN;
 use warnings;
 use strict;
 use Carp;
-use File::Spec::Functions qw/catfile catdir/;
+use File::Spec;
 use Shipwright::Source::Compressed;
 use CPAN;
 use Data::Dumper;
 use File::Temp qw/tempdir/;
+use File::Spec;
 use File::Slurp;
+use UNIVERSAL::require;
 use CPAN::DistnameInfo;
 
 use base qw/Shipwright::Source::Base/;
 
-my $cpan_dir = tempdir( 'shipwright_XXXXXX', CLEANUP => 1, TMPDIR => 1 );
+my $cpan_dir = tempdir( CLEANUP => 0 );
 unshift @INC, $cpan_dir;
 
 =head1 NAME
@@ -30,24 +32,19 @@ sub new {
     my $class = shift;
     my $self  = $class->SUPER::new(@_);
 
-    require Module::Info;
-    if ( Module::Info->new_from_module('CPAN::Config') ) {
+    CPAN::Config->use;
 
-        # keep original CPAN::Config info
-        require CPAN::Config;
-    }
-
-    mkdir catdir( $cpan_dir, 'CPAN' );
-    my $config_file = catfile( $cpan_dir, 'CPAN', 'MyConfig.pm' );
+    mkdir File::Spec->catfile( $cpan_dir, 'CPAN' );
+    my $config_file = File::Spec->catfile( $cpan_dir, 'CPAN', 'MyConfig.pm' );
 
     unless ( -f $config_file ) {
-
-        # hack $CPAN::Config, mostly to make cpan stuff temporary
-        $CPAN::Config->{cpan_home}         = catdir($cpan_dir);
-        $CPAN::Config->{build_dir}         = catdir( $cpan_dir, 'build' );
-        $CPAN::Config->{histfile}          = catfile( $cpan_dir, 'histfile' );
-        $CPAN::Config->{keep_source_where} = catdir( $cpan_dir, 'sources' );
-        $CPAN::Config->{prefs_dir}         = catdir( $cpan_dir, 'prefs' );
+        $CPAN::Config->{cpan_home} = File::Spec->catfile($cpan_dir);
+        $CPAN::Config->{build_dir} = File::Spec->catfile( $cpan_dir, 'build' );
+        $CPAN::Config->{histfile} =
+          File::Spec->catfile( $cpan_dir, 'histfile' );
+        $CPAN::Config->{keep_source_where} =
+          File::Spec->catfile( $cpan_dir, 'sources' );
+        $CPAN::Config->{prefs_dir} = File::Spec->catfile( $cpan_dir, 'prefs' );
         $CPAN::Config->{prerequisites_policy} = 'follow';
         $CPAN::Config->{urllist}              = [];
         write_file( $config_file,
@@ -71,7 +68,7 @@ sub run {
         $compressed->run(@_);
     }
     else {
-        confess 'invalid source: ' . $self->source;
+        die 'invalid source: ' . $self->source;
     }
 }
 
@@ -81,7 +78,7 @@ sub _run {
 
     my ( $source, $distribution );
 
-    Shipwright::Util->select('cpan');
+    Shipwright::Util->select( 'cpan' );
 
     if ( $self->source =~ /\.tar\.gz$/ ) {
 
@@ -132,19 +129,23 @@ sub _run {
 
     my $name = CPAN::DistnameInfo->new( $distribution->{ID} )->dist;
 
+
     if ( $name eq 'perl' ) {
-        confess 'perl itself contains ' . $self->source . ', will not process';
+        die 'perl itself contains ' . $self->source . ', will not process';
     }
 
     $distribution->get;
 
-    Shipwright::Util->select('stdout');
+    Shipwright::Util->select( 'stdout' );
 
     $self->name( 'cpan-' . $name );
     $self->_update_map( $self->source, 'cpan-' . $name );
 
     $self->source(
-        catfile( $CPAN::Config->{keep_source_where}, 'authors', 'id', $source )
+        File::Spec->catfile(
+            $CPAN::Config->{keep_source_where},
+            'authors', 'id', $source
+        )
     );
     return 1;
 }
