@@ -3,7 +3,7 @@ package Shipwright::Source::SVN;
 use warnings;
 use strict;
 use Carp;
-use File::Spec::Functions qw/catfile catdir/;
+use File::Spec;
 
 use base qw/Shipwright::Source::Base/;
 
@@ -52,7 +52,7 @@ sub _run {
 
     my $cmd    = [
         'svn', 'export', $self->source,
-        catfile( $self->download_directory, $self->name ),
+        File::Spec->catfile( $self->download_directory, $self->name ),
         $self->version ? ( '-r', $self->version ) : (),
     ];
 
@@ -65,7 +65,7 @@ sub _run {
     }
 
     $self->source(
-        catfile( $self->download_directory, $self->name ) );
+        File::Spec->catfile( $self->download_directory, $self->name ) );
     Shipwright::Util->run($cmd);
 }
 
